/*
 * Copyright(C) 2012 - 2013 orinos Co.,Ltd. All rights reserved.
 * http://www.orinos.co.jp/
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.aimluck.eip.modules.screens;

import net.sf.json.JSONArray;

import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

import com.aimluck.eip.common.ALEipConstants;
import com.aimluck.eip.improveope.ImproveopeActionFormData;
import com.aimluck.eip.improveope.ImproveopeApplyFormData;
import com.aimluck.eip.improveope.ImproveopeCheckFormData;
import com.aimluck.eip.improveope.ImproveopeReceiveFormData;
import com.aimluck.eip.improveope.ImproveopeReportFormData;
import com.aimluck.eip.improveope.ImproveopeTopicMultiDelete;

/**
 * 業務改善申請をJSONデータとして出力するクラスです。
 *
 */
public class ImproveopeApplyFormJSONScreen extends ALJSONScreen {

  /** logger */
  private static final JetspeedLogger logger = JetspeedLogFactoryService
    .getLogger(ImproveopeApplyFormJSONScreen.class.getName());

  @Override
  protected String getJSONString(RunData rundata, Context context)
      throws Exception {
    String result = new JSONArray().toString();
    String mode = this.getMode();
    try {

      if (ALEipConstants.MODE_INSERT.equals(mode)) {
        //
    	ImproveopeApplyFormData formData = new ImproveopeApplyFormData();
        formData.initField();
        if (formData.doInsert(this, rundata, context)) {
        } else {
          JSONArray json =
            JSONArray
              .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
          result = json.toString();
        }

      } else if (ALEipConstants.MODE_UPDATE.equals(mode)) {

    	ImproveopeApplyFormData formData = new ImproveopeApplyFormData();
        formData.initField();
        if (formData.doUpdate(this, rundata, context)) {
        } else {
          JSONArray json =
            JSONArray
              .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
          result = json.toString();
        }
      } else if (ALEipConstants.MODE_DELETE.equals(mode)) {

    	ImproveopeApplyFormData formData = new ImproveopeApplyFormData();
        formData.initField();
        if (formData.doDelete(this, rundata, context)) {
        } else {
          JSONArray json =
            JSONArray
              .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
          result = json.toString();
        }
      } else if (ALEipConstants.MODE_MULTI_DELETE.equals(mode)) {
        ImproveopeTopicMultiDelete delete = new ImproveopeTopicMultiDelete();
        if (delete.doMultiAction(this, rundata, context)) {
        } else {
          JSONArray json =
            JSONArray
              .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
          result = json.toString();
        }
      } else if ("receive".equals(mode)) {
    	  ImproveopeReceiveFormData receive = new ImproveopeReceiveFormData();
    	  receive.initField();
          if (receive.doUpdate(this, rundata, context)) {
          } else {
            JSONArray json =
              JSONArray
                .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
            result = json.toString();
          }
      } else if ("checkinsert".equals(mode)) {
    	  ImproveopeCheckFormData check = new ImproveopeCheckFormData();
    	  check.initField();
          if (check.doInsert(this, rundata, context)) {
          } else {
            JSONArray json =
              JSONArray
                .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
            result = json.toString();
          }
      } else if ("checkupdate".equals(mode)) {
    	  ImproveopeCheckFormData check = new ImproveopeCheckFormData();
    	  check.initField();
          if (check.doUpdate(this, rundata, context)) {
          } else {
            JSONArray json =
              JSONArray
                .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
            result = json.toString();
          }
      } else if ("actioninsert".equals(mode)) {
    	  ImproveopeActionFormData check = new ImproveopeActionFormData();
    	  check.initField();
          if (check.doInsert(this, rundata, context)) {
          } else {
            JSONArray json =
              JSONArray
                .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
            result = json.toString();
          }
      } else if ("actionupdate".equals(mode)) {
    	  ImproveopeActionFormData check = new ImproveopeActionFormData();
    	  check.initField();
          if (check.doUpdate(this, rundata, context)) {
          } else {
            JSONArray json =
              JSONArray
                .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
            result = json.toString();
          }
      } else if ("actiondelete".equals(mode)) {

    	  ImproveopeActionFormData formData = new ImproveopeActionFormData();
        formData.initField();
        if (formData.doDelete(this, rundata, context)) {
        } else {
          JSONArray json =
            JSONArray
              .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
          result = json.toString();
        }
      } else if ("reportupdate".equals(mode)) {
    	  ImproveopeReportFormData report = new ImproveopeReportFormData();
    	  report.initField();
          if (report.doUpdate(this, rundata, context)) {
          } else {
            JSONArray json =
              JSONArray
                .fromObject(context.get(ALEipConstants.ERROR_MESSAGE_LIST));
            result = json.toString();
          }
        }

    } catch (Exception e) {
      logger.error("[ImproveopeTopicFormJSONScreen]", e);
    }

    return result;
  }
}
